/*  incomplete DUMB JGMOD compatibility wrapper, because JGMOD's license is
    not opensource / GPL compatible and because there are already enough mod
    playing libs.
   
    Copyright 2006-2008 Hans de Goede <j.w.r.degoede@hhs.nl>
   
    This wrapper is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public   
    License as published by the Free Software Foundation; either 
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of 
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA   
*/
#include <unistd.h>
#include <pthread.h>
#include <string.h>
#include <strings.h>
#include "jgmod.h"

static AL_DUH_PLAYER *duh_player = NULL;
static pthread_t duh_player_thread;
static pthread_mutex_t duh_player_mutex;
static int pattern = 0; /* from t3ss implementation, might need it someday */
static int mod_volume = 255;

int install_mod(int channels) /* note: channels is ignored */
{
    dumb_register_packfiles();
    pthread_mutex_init(&duh_player_mutex, NULL);
    
    return 0;
}

void remove_mod(void)
{
    stop_mod();
    dumb_exit();
}

JGMOD *load_mod(const char *path) /* path may be a packfile # index */
{
    char *p, ext[64];
    JGMOD *mod;

    snprintf(ext, sizeof(ext), "%s", get_extension(path));
    if ((p = strrchr(ext, '_')))
        memmove(ext, p+1, strlen(p));
    
    if (!strcasecmp(ext, "it"))
        return dumb_load_it_quick(path);
    if (!strcasecmp(ext, "xm"))
        return dumb_load_xm_quick(path);
    if (!strcasecmp(ext, "s3m"))
        return dumb_load_s3m_quick(path);
    if (!strcasecmp(ext, "mod"))
        return dumb_load_mod_quick(path);
        
    /* Darn opening by extension didn't succeed, try determining the filetype
       by content, start with xm, as that has a very strong signature */
    if ((mod = dumb_load_xm_quick(path)))
        return mod;
        
    /* next try it (4 byte signature) */
    if ((mod = dumb_load_it_quick(path)))
        return mod;
        
    /* next try s3m (2 byte signature) */
    if ((mod = dumb_load_s3m_quick(path)))
        return mod;
        
    /* Hmm mod doesn't have a real signature, but it does have several length
       fields, so loading will still most likely fail, besides that it cannot
       be much else */
    return dumb_load_mod_quick(path);
}

void destroy_mod(JGMOD *mod)
{
    unload_duh(mod);
}

static void *duh_player_thread_function(void *arg)
{
    while(1)
    {
        pthread_mutex_lock(&duh_player_mutex);
        al_poll_duh(duh_player);
        pthread_mutex_unlock(&duh_player_mutex);
        pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
        usleep(10000);
        pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, NULL);
    }
}

void play_mod(JGMOD *mod, int loop)
{
    DUMB_IT_SIGDATA *sigdata;
    DUH_SIGRENDERER *renderer;
    int num_patterns;
    
    if (duh_player)
        stop_mod();
    
    if (!mod)
        return;
    
    sigdata = duh_get_it_sigdata(mod);
    if (sigdata)
        num_patterns = dumb_it_sd_get_n_orders(sigdata);
    else
        num_patterns = 0;
        
    if (pattern >= num_patterns)
        pattern = 0;
        
    renderer = dumb_it_start_at_order(mod, 2, pattern);
    if (!renderer)
        return;

    duh_player = al_duh_encapsulate_sigrenderer(renderer, mod_volume/255.0f,
        2048, 44100);
    if (!duh_player)
    {
        duh_end_sigrenderer(renderer);
        return;
    }
    
    if (pthread_create(&duh_player_thread, NULL,
            duh_player_thread_function, NULL))
    {
        al_stop_duh(duh_player);
        duh_player = NULL;
    }
}


void stop_mod(void)
{
    if (duh_player)
    {
        /* stop the player thread */
        pthread_cancel(duh_player_thread);
        pthread_join(duh_player_thread, NULL);
        /* and clean up everything */
        al_stop_duh(duh_player);
        duh_player = NULL;
    }
}

void set_mod_volume(int volume) /* 0 - 255 */
{
    mod_volume = volume;
    if (duh_player)
    {
        pthread_mutex_lock(&duh_player_mutex);
        al_duh_set_volume(duh_player, mod_volume/255.0f);
        pthread_mutex_unlock(&duh_player_mutex);
    }
}
